
local function updateDirectSowingAreaNoTill(fruitIndex, superFunc, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, fieldGroundType, angle, growthState, blockedSprayTypeIndex)
	-- TerraLiveExtension.DebugText("NoTillFSDensityMapUtil:updateDirectSowingArea(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)", fruitIndex, superFunc, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, fieldGroundType, angle, growthState, blockedSprayTypeIndex);
	-- replace the original with a version which could not direct seed in higher fruits
	local fruitTypeManager = g_fruitTypeManager
	local desc = fruitTypeManager:getFruitTypeByIndex(fruitIndex)

	if desc.terrainDataPlaneId == nil then
		return 0, 0
	end

	angle = angle or 0
	growthState = growthState or 1
	local functionData = FSDensityMapUtil.functionCache.updateDirectSowingArea

	if functionData == nil then
		local terrainRootNode = g_currentMission.terrainRootNode
		local fieldGroundSystem = g_currentMission.fieldGroundSystem
		local groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_TYPE)
		local groundAngleMapId, groundAngleFirstChannel, groundAngleNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_ANGLE)
		local sprayLevelMapId, sprayLevelFirstChannel, sprayLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.SPRAY_LEVEL)
		local sprayLevelMaxValue = fieldGroundSystem:getMaxValue(FieldDensityMap.SPRAY_LEVEL)
		local sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.SPRAY_TYPE)
		local sownType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.SOWN)
		local directSownType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.DIRECT_SOWN)
		local ridgeType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.RIDGE)
		local stubbleTillagedType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.STUBBLE_TILLAGE)
		local firstSowableValue, lastSowableValue = fieldGroundSystem:getSowableRange()
		local firstSowingValue, lastSowingValue = fieldGroundSystem:getSowingRange()
		local rollerLevelMapId, rollerLevelFirstChannel, rollerLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.ROLLER_LEVEL)
		local rollerLevelMaxValue = fieldGroundSystem:getMaxValue(FieldDensityMap.ROLLER_LEVEL)
		functionData = {
			multiModifiers = {},
			fruitModifiers = {},
			fruitFilters = {},
			sprayLevelModifier = DensityMapModifier.new(sprayLevelMapId, sprayLevelFirstChannel, sprayLevelNumChannels, terrainRootNode),
			sprayLevelFilter = DensityMapFilter.new(sprayLevelMapId, sprayLevelFirstChannel, sprayLevelNumChannels)
		}

		functionData.sprayLevelFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 0, sprayLevelMaxValue - 1)

		functionData.sprayTypeModifier = DensityMapModifier.new(sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels, terrainRootNode)
		functionData.groundTypeModifier = DensityMapModifier.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels, terrainRootNode)
		functionData.groundAngleModifier = DensityMapModifier.new(groundAngleMapId, groundAngleFirstChannel, groundAngleNumChannels, terrainRootNode)
		functionData.fieldFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)

		functionData.fieldFilter:setValueCompareParams(DensityValueCompareType.GREATER, 0)

		functionData.sowableFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)

		functionData.sowableFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, firstSowableValue, lastSowableValue)

		functionData.sowingFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels, terrainRootNode)

		functionData.sowingFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, firstSowingValue, lastSowingValue)

		functionData.stubbleTillageFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels, terrainRootNode)

		functionData.stubbleTillageFilter:setValueCompareParams(DensityValueCompareType.EQUAL, stubbleTillagedType)

		functionData.directSownFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)

		functionData.directSownFilter:setValueCompareParams(DensityValueCompareType.EQUAL, directSownType)

		functionData.sownFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)

		functionData.sownFilter:setValueCompareParams(DensityValueCompareType.EQUAL, sownType)

		if Platform.gameplay.useRolling then
			functionData.rollerLevelModifier = DensityMapModifier.new(rollerLevelMapId, rollerLevelFirstChannel, rollerLevelNumChannels, terrainRootNode)
			functionData.rollerLevelMaxValue = rollerLevelMaxValue
		end

		functionData.sownType = sownType
		functionData.directSownType = directSownType
		functionData.ridgeType = ridgeType
		FSDensityMapUtil.functionCache.updateDirectSowingArea = functionData
	end

	local sowableFilter = functionData.sowableFilter
	local sowingFilter = functionData.sowingFilter
	local fieldFilter = functionData.fieldFilter
	local ridgeType = functionData.ridgeType
	local directSownType = functionData.directSownType
	local stubbleTillageFilter = functionData.stubbleTillageFilter
	local directSownFilter = functionData.directSownFilter
	local sownFilter = functionData.sownFilter
	local sprayLevelModifier = functionData.sprayLevelModifier
	local sprayLevelFilter = functionData.sprayLevelFilter
	local sprayTypeModifier = functionData.sprayTypeModifier
	local groundTypeModifier = functionData.groundTypeModifier
	local groundAngleModifier = functionData.groundAngleModifier
	local fruitMultiModifier = functionData.multiModifiers[fruitIndex]
	local fruitModifier = functionData.fruitModifiers[fruitIndex]
	local terrainRootNode = g_currentMission.terrainRootNode
	local rollerLevelModifier = functionData.rollerLevelModifier
	local rollerLevelMaxValue = functionData.rollerLevelMaxValue

	if fruitMultiModifier == nil then
		fruitMultiModifier = {}
		functionData.multiModifiers[fruitIndex] = fruitMultiModifier
		fruitModifier = DensityMapModifier.new(desc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
		functionData.fruitModifiers[fruitIndex] = fruitModifier
	end

	local multiModifier = fruitMultiModifier[growthState]

	if multiModifier == nil then
		local fieldGroundSystem = g_currentMission.fieldGroundSystem
		local stubbleTillagedType = fieldGroundSystem:getFieldGroundValue(FieldGroundType.STUBBLE_TILLAGE)
		multiModifier = DensityMapMultiModifier.new()
		fruitMultiModifier[growthState] = multiModifier

		for index, fruitDesc in pairs(fruitTypeManager:getFruitTypes()) do
			if fruitDesc.terrainDataPlaneId ~= nil and fruitDesc.destruction.canBeDestroyed then
				local fruitFilter = functionData.fruitFilters[index]

				if fruitFilter == nil then
					fruitFilter = DensityMapFilter.new(fruitDesc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
					functionData.fruitFilters[index] = fruitFilter
				end

				if fruitDesc.cutState > 1 then
					fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 2, fruitDesc.cutState - 1)
					multiModifier:addExecuteAdd(1, sprayLevelModifier, sprayLevelFilter, fieldFilter, fruitFilter)
					multiModifier:addExecuteSet(1, sprayTypeModifier, fieldFilter, fruitFilter)
				end

				local maxState = fruitDesc.maxWeederHoeState;
				if maxState == nil or maxState == 0 then
					maxState = fruitDesc.groundTypeChangeGrowthState;
				end
				local mulcherState = fruitDesc.mulcher.state or fruitDesc.cutState;
				
				if fruitDesc.allowsSeeding then
					if index ~= fruitIndex then
						-- fruitFilter:setValueCompareParams(DensityValueCompareType.GREATER, 0)
						-- multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
						fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 0, maxState)
						multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
						
						fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, fruitDesc.cutState, mulcherState)
						multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
					else
						-- fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, growthState + 1, math.max(fruitDesc.maxHarvestingGrowthState, fruitDesc.cutState))
						-- multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
						fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, growthState + 1, maxState)
						multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
					end

					if fruitDesc.mulcher.state ~= nil and fruitDesc.mulcher.state > 0 and fruitDesc.mulcher.state ~= growthState then
						fruitFilter:setValueCompareParams(DensityValueCompareType.EQUAL, fruitDesc.mulcher.state)
						multiModifier:addExecuteSet(stubbleTillagedType, groundTypeModifier, fieldFilter, fruitFilter)
					end
				end

				local modifier = functionData.fruitModifiers[index]

				if modifier == nil then
					modifier = DensityMapModifier.new(fruitDesc.terrainDataPlaneId, desc.startStateChannel, desc.numStateChannels, terrainRootNode)
					functionData.fruitModifiers[index] = modifier
				end

				if index ~= fruitIndex then
					-- fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 2, math.max(fruitDesc.maxHarvestingGrowthState, fruitDesc.cutState))
					fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, 2, maxState);
					multiModifier:addExecuteSet(0, modifier, fruitFilter, fieldFilter)
					
					fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, fruitDesc.cutState, mulcherState);
					multiModifier:addExecuteSet(0, modifier, fruitFilter, fieldFilter)
				else
					-- fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, growthState + 1, math.max(fruitDesc.maxHarvestingGrowthState, fruitDesc.cutState))
					fruitFilter:setValueCompareParams(DensityValueCompareType.BETWEEN, growthState + 1, maxState);
					multiModifier:addExecuteSet(0, modifier, fruitFilter, fieldFilter)
				end

			end
		end
	end

	fruitModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
	groundTypeModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
	groundAngleModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
	multiModifier:updateParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
	multiModifier:execute(false)

	if g_currentMission.missionInfo.weedsEnabled and desc.plantsWeed then
		FSDensityMapUtil.setSowingWeedArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)
	else
		FSDensityMapUtil.removeWeedArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ)
	end

	if Platform.gameplay.useRolling then
		rollerLevelModifier:setParallelogramWorldCoords(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, DensityCoordType.POINT_POINT_POINT)

		local rollerLevelValue = 0

		if desc.needsRolling then
			rollerLevelValue = rollerLevelMaxValue
		end

		rollerLevelModifier:executeSet(rollerLevelValue, sowableFilter)
	end

	local _, changedArea, totalArea = fruitModifier:executeSet(growthState, sowableFilter)

	if fieldGroundType ~= ridgeType then
		groundTypeModifier:executeSet(directSownType, sowableFilter, stubbleTillageFilter)

		local _, directArea, totalSownArea = groundTypeModifier:executeGet(directSownFilter)

		if totalSownArea > 0 and directArea / totalSownArea > 0.5 then
			groundTypeModifier:executeSet(directSownType, sownFilter)
		end
	end

	groundTypeModifier:executeSet(fieldGroundType, sowableFilter)
	groundAngleModifier:executeSet(angle, sowingFilter)
	FSDensityMapUtil.removeSprayArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, blockedSprayTypeIndex)
	DensityMapHeightUtil.clearArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ)

	return changedArea, totalArea
end

FSDensityMapUtil.updateDirectSowingArea = Utils.overwrittenFunction(FSDensityMapUtil.updateDirectSowingArea, updateDirectSowingAreaNoTill)